// StopLossAgent.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

/**
 *StopLossAgent inherits from BasicSumAgent and operates in random way,
 *like a random agent. 
 *After that, its behavior can be modified by a "stop loss"
 *decision, operating in two possible ways 
 *(i) without memory:
 *    if the current price, i.e. the last executedPrice (at day t) is
 *      >= to (mean price at day t-stopLossInterval) *(1+maxLossRate)
 *      <= to (mean price at day t-stopLossInterval) *(1-maxLossRate)
 *      and checkingIfShortOrLong=0,
 *    the agent buys/sells, at the current price, a quantity of shares between 1 
 *    and maxOrderQuantity;
 *
 *(ii) with memory (the short or long position of the agent is taken in account; if
 *     the agent has no position, doesn't operate here):
 *     if the current price, i.e. the last executedPrice (at day t) is
 *       >= to (mean price at day t-stopLossInterval) *(1+maxLossRate)
 *       <= to (mean price at day t-stopLossInterval) *(1-maxLossRate)
 *       and checkingIfShortOrLong=1,
 *    the agent buys  if short (shareQuantity<0);
 *              sells if long  (shareQuantity>0)
 *    at the current price, a quantity of shares between 1 
 *    and maxOrderQuantity  (regardless its shareQuantity value).
 *
 *
 *@author Bruno Mencarelli integration of Antonio de Ruvo
 *
 */

public class StopLossAgent extends BasicSumAgent
        {
        /** 
         *This is the vector containing the basic input of an order:
         *the price of the offer, the quantity of shares and the identification number of the agent.
         *
         */ 
        public double[] order= new double[4];

        /**These are the variables that the agent checks for the stop-loss strategy. */
        int stopLossInterval,checkingIfShortOrLong;
        /**THis is the maximun rate of loss that the agent will accept before appliying the strategy.*/
        double maxLossRate;

        /**This is the RuleMaster of the stop loss agent. */
        public StopLossRuleMaster ruleMaster;

        /** Constructor for a new stop loss agent. */
        public StopLossAgent(Zone aZone, int maxOrderQuantity)
                {
                super(aZone, maxOrderQuantity);
                }
  
                /**This method stes the Rule Master. */
                public void setRuleMaster(StopLossRuleMaster r)
                {
                ruleMaster=r;
                }

                /**This method sets the stoplossinterval. */
                public void setStopLossInterval(int i)
                {
                stopLossInterval=i;
                }

                /**This method sets the maxlossrate and the option checking if short or long. */
                public void setMaxLossRate$andCheckingIfShortOrLong(double r,int sl)
                {
                maxLossRate=r;
                checkingIfShortOrLong=sl;
                }

        /**This method is called in the first phase of the day. */
        public void act0()
                {
                double lastPrice,stopLossMeanPrice;
                int i;

                buySellSwitch=0.5;//random choice

                lastPrice=theBook.getPrice();
                //stopLossMeanPrice assumes the mean value at the time t-stopLossInterval called back from the book                              
                stopLossMeanPrice=theBook.getLaggedMeanPrice(stopLossInterval); 
                price = ruleMaster.getPriceInOpeningOrClosingAuctions (lastPrice, buySellSwitch, stopLossMeanPrice,maxLossRate,shareQuantity,checkingIfShortOrLong);
                if (maxOrderQuantity==1) 
                        iMax=1;
                else
                        iMax=Globals.env.uniformIntRand.getIntegerWithMin$withMax(1, maxOrderQuantity); 

                
                if (price!=0 && agentProbToActWithMarketPrice > Globals.env.uniformDblRand
		    .getDoubleWithMin$withMax(0.0, 1.0)){
		   //sell
		   if(price<0){
		       order[0]=0;
		       order[3]=2;
		   }
		   //buy
		   else if(price>0){
		       order[0]=0;
		       order[3]=1;
		   }
		}
		else {
		  order[0]=price;
		  order[3]=0;
		}
                order[1]=number;
                order[2]=iMax;
		if(printing==2){
                  System.out.println("I'm the stopLossAgent "+order[1]+" and I sent an order in act0 for "
                                     +order[2]+" shares at price"+order[0]);
		}		  
                theBook.setOrderInOpeningOrClosingAuctionsFromAgent (order);
                }

        /**
         *This method is called during the simulated day. 
         *The agent acts in the market.
         */ 
        public void act1()
                {
                double lastPrice,stopLossMeanPrice;
                int i;
                buySellSwitch=0.5;
                lastPrice=theBook.getPrice();
                stopLossMeanPrice=theBook.getLaggedMeanPrice(stopLossInterval); 
                price = ruleMaster.getPrice (lastPrice, buySellSwitch, stopLossMeanPrice,maxLossRate,shareQuantity,checkingIfShortOrLong);
                        if (maxOrderQuantity==1) 
                                iMax=1;
                        else
                                iMax=Globals.env.uniformIntRand.getIntegerWithMin$withMax(1, maxOrderQuantity); 
                if (price!=0 && agentProbToActWithMarketPrice > Globals.env.uniformDblRand
		    .getDoubleWithMin$withMax(0.0, 1.0)){
		   //sell
		   if(price<0){
		       order[0]=0;
		       order[3]=2;
		   }
		   //buy
		   else if(price>0){
		       order[0]=0;
		       order[3]=1;
		   }
		}
		else {
		  order[0]=price;
		  order[3]=0;
		}
                        order[1]=number;
                        order[2]=iMax;
			if(printing==2){
                          System.out.println("I'm the stopLossAgent "+order[1]+" and I sent an order in act1 for "
                                             +order[2]+" shares at price"+order[0]);
			}
                        theBook.setOrderFromAgent (order);                
                }
        }



